import * as puppeteer from 'puppeteer';
import { ElementHandle, Page, JSHandle } from 'puppeteer';
export { ElementHandle, JSHandle } from 'puppeteer';
import * as esbuild from 'esbuild';
import { Plugin as Plugin$1 } from 'rollup';
import axe from 'axe-core';

interface Plugin extends Plugin$1 {
    enforce?: 'pre' | 'normal' | 'post';
}

interface ModuleServerOpts {
    root?: string;
    /** List of Rollup/Vite/WMR plugins to add */
    plugins?: (Plugin | false | undefined)[];
    /**
     * Environment variables to pass into the bundle.
     * They can be accessed via import.meta.env.<name> (or process.env.<name> for compatability)
     */
    envVars?: Record<string, string>;
    /** Options to pass to esbuild. Set to false to disable esbuild */
    esbuild?: esbuild.TransformOptions | false;
}

type ARIAWidgetRole =
    | 'button'
    | 'checkbox'
    | 'gridcell'
    | 'link'
    | 'menuitem'
    | 'menuitemcheckbox'
    | 'menuitemradio'
    | 'option'
    | 'progressbar'
    | 'radio'
    | 'scrollbar'
    | 'searchbox'
    | 'slider'
    | 'spinbutton'
    | 'switch'
    | 'tab'
    | 'tabpanel'
    | 'textbox'
    | 'treeitem';

type ARIACompositeWidgetRole =
    | 'combobox'
    | 'grid'
    | 'listbox'
    | 'menu'
    | 'menubar'
    | 'radiogroup'
    | 'tablist'
    | 'tree'
    | 'treegrid';

type ARIADocumentStructureRole =
    | 'application'
    | 'article'
    | 'blockquote'
    | 'caption'
    | 'cell'
    | 'columnheader'
    | 'definition'
    | 'deletion'
    | 'directory'
    | 'document'
    | 'emphasis'
    | 'feed'
    | 'figure'
    | 'generic'
    | 'group'
    | 'heading'
    | 'img'
    | 'insertion'
    | 'list'
    | 'listitem'
    | 'math'
    | 'meter'
    | 'none'
    | 'note'
    | 'paragraph'
    | 'presentation'
    | 'row'
    | 'rowgroup'
    | 'rowheader'
    | 'separator'
    | 'strong'
    | 'subscript'
    | 'superscript'
    | 'table'
    | 'term'
    | 'time'
    | 'toolbar'
    | 'tooltip';

type ARIALandmarkRole =
    | 'banner'
    | 'complementary'
    | 'contentinfo'
    | 'form'
    | 'main'
    | 'navigation'
    | 'region'
    | 'search';

type ARIALiveRegionRole = 'alert' | 'log' | 'marquee' | 'status' | 'timer';

type ARIAWindowRole = 'alertdialog' | 'dialog';

type ARIAUncategorizedRole = 'code';

type ARIARole =
    | ARIAWidgetRole
    | ARIACompositeWidgetRole
    | ARIADocumentStructureRole
    | ARIALandmarkRole
    | ARIALiveRegionRole
    | ARIAWindowRole
    | ARIAUncategorizedRole;

type MatcherFunction = (
  content: string,
  element: Element | null,
) => boolean
type Matcher = MatcherFunction | RegExp | number | string

// Get autocomplete for ARIARole union types, while still supporting another string
// Ref: https://github.com/microsoft/TypeScript/issues/29729#issuecomment-505826972
type ByRoleMatcher = ARIARole | MatcherFunction | {}

type NormalizerFn = (text: string) => string

interface MatcherOptions {
  exact?: boolean
  /** Use normalizer with getDefaultNormalizer instead */
  trim?: boolean
  /** Use normalizer with getDefaultNormalizer instead */
  collapseWhitespace?: boolean
  normalizer?: NormalizerFn
  /** suppress suggestions for a specific query */
  suggest?: boolean
}

interface waitForOptions {
  container?: HTMLElement
  timeout?: number
  interval?: number
  onTimeout?: (error: Error) => Error
  mutationObserverOptions?: MutationObserverInit
}

interface SelectorMatcherOptions extends MatcherOptions {
  selector?: string
  ignore?: boolean | string
}

type QueryByBoundAttribute<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: MatcherOptions,
) => T | null

type AllByBoundAttribute<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: MatcherOptions,
) => T[]

type FindAllByBoundAttribute<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: MatcherOptions,
  waitForElementOptions?: waitForOptions,
) => Promise<T[]>

type GetByBoundAttribute<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: MatcherOptions,
) => T

type FindByBoundAttribute<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: MatcherOptions,
  waitForElementOptions?: waitForOptions,
) => Promise<T>

type QueryByText<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: SelectorMatcherOptions,
) => T | null

type AllByText<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: SelectorMatcherOptions,
) => T[]

type FindAllByText<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: SelectorMatcherOptions,
  waitForElementOptions?: waitForOptions,
) => Promise<T[]>

type GetByText<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: SelectorMatcherOptions,
) => T

type FindByText<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  id: Matcher,
  options?: SelectorMatcherOptions,
  waitForElementOptions?: waitForOptions,
) => Promise<T>

interface ByRoleOptions extends MatcherOptions {
  /**
   * If true includes elements in the query set that are usually excluded from
   * the accessibility tree. `role="none"` or `role="presentation"` are included
   * in either case.
   */
  hidden?: boolean
  /**
   * If true only includes elements in the query set that are marked as
   * selected in the accessibility tree, i.e., `aria-selected="true"`
   */
  selected?: boolean
  /**
   * If true only includes elements in the query set that are marked as
   * checked in the accessibility tree, i.e., `aria-checked="true"`
   */
  checked?: boolean
  /**
   * If true only includes elements in the query set that are marked as
   * pressed in the accessibility tree, i.e., `aria-pressed="true"`
   */
  pressed?: boolean
  /**
   * Filters elements by their `aria-current` state. `true` and `false` match `aria-current="true"` and `aria-current="false"` (as well as a missing `aria-current` attribute) respectively.
   */
  current?: boolean | string
  /**
   * If true only includes elements in the query set that are marked as
   * expanded in the accessibility tree, i.e., `aria-expanded="true"`
   */
  expanded?: boolean
  /**
   * Includes elements with the `"heading"` role matching the indicated level,
   * either by the semantic HTML heading elements `<h1>-<h6>` or matching
   * the `aria-level` attribute.
   */
  level?: number
  /**
   * Includes every role used in the `role` attribute
   * For example *ByRole('progressbar', {queryFallbacks: true})` will find <div role="meter progressbar">`.
   */
  queryFallbacks?: boolean
  /**
   * Only considers elements with the specified accessible name.
   */
  name?:
    | RegExp
    | string
    | ((accessibleName: string, element: Element) => boolean)
  /**
   * Only considers elements with the specified accessible description.
   */
  description?:
    | RegExp
    | string
    | ((accessibleDescription: string, element: Element) => boolean)
}

type AllByRole<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  role: ByRoleMatcher,
  options?: ByRoleOptions,
) => T[]

type GetByRole<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  role: ByRoleMatcher,
  options?: ByRoleOptions,
) => T

type QueryByRole<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  role: ByRoleMatcher,
  options?: ByRoleOptions,
) => T | null

type FindByRole<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  role: ByRoleMatcher,
  options?: ByRoleOptions,
  waitForElementOptions?: waitForOptions,
) => Promise<T>

type FindAllByRole<T extends HTMLElement = HTMLElement> = (
  container: HTMLElement,
  role: ByRoleMatcher,
  options?: ByRoleOptions,
  waitForElementOptions?: waitForOptions,
) => Promise<T[]>

declare function getByLabelText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByText<T>>
): ReturnType<GetByText<T>>
declare function getAllByLabelText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByText<T>>
): ReturnType<AllByText<T>>
declare function queryByLabelText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByText<T>>
): ReturnType<QueryByText<T>>
declare function queryAllByLabelText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByText<T>>
): ReturnType<AllByText<T>>
declare function findByLabelText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByText<T>>
): ReturnType<FindByText<T>>
declare function findAllByLabelText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByText<T>>
): ReturnType<FindAllByText<T>>
declare function getByPlaceholderText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByBoundAttribute<T>>
): ReturnType<GetByBoundAttribute<T>>
declare function getAllByPlaceholderText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function queryByPlaceholderText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByBoundAttribute<T>>
): ReturnType<QueryByBoundAttribute<T>>
declare function queryAllByPlaceholderText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function findByPlaceholderText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByBoundAttribute<T>>
): ReturnType<FindByBoundAttribute<T>>
declare function findAllByPlaceholderText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByBoundAttribute<T>>
): ReturnType<FindAllByBoundAttribute<T>>
declare function getByText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByText<T>>
): ReturnType<GetByText<T>>
declare function getAllByText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByText<T>>
): ReturnType<AllByText<T>>
declare function queryByText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByText<T>>
): ReturnType<QueryByText<T>>
declare function queryAllByText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByText<T>>
): ReturnType<AllByText<T>>
declare function findByText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByText<T>>
): ReturnType<FindByText<T>>
declare function findAllByText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByText<T>>
): ReturnType<FindAllByText<T>>
declare function getByAltText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByBoundAttribute<T>>
): ReturnType<GetByBoundAttribute<T>>
declare function getAllByAltText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function queryByAltText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByBoundAttribute<T>>
): ReturnType<QueryByBoundAttribute<T>>
declare function queryAllByAltText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function findByAltText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByBoundAttribute<T>>
): ReturnType<FindByBoundAttribute<T>>
declare function findAllByAltText<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByBoundAttribute<T>>
): ReturnType<FindAllByBoundAttribute<T>>
declare function getByTitle<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByBoundAttribute<T>>
): ReturnType<GetByBoundAttribute<T>>
declare function getAllByTitle<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function queryByTitle<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByBoundAttribute<T>>
): ReturnType<QueryByBoundAttribute<T>>
declare function queryAllByTitle<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function findByTitle<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByBoundAttribute<T>>
): ReturnType<FindByBoundAttribute<T>>
declare function findAllByTitle<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByBoundAttribute<T>>
): ReturnType<FindAllByBoundAttribute<T>>
declare function getByDisplayValue<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByBoundAttribute<T>>
): ReturnType<GetByBoundAttribute<T>>
declare function getAllByDisplayValue<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function queryByDisplayValue<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByBoundAttribute<T>>
): ReturnType<QueryByBoundAttribute<T>>
declare function queryAllByDisplayValue<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function findByDisplayValue<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByBoundAttribute<T>>
): ReturnType<FindByBoundAttribute<T>>
declare function findAllByDisplayValue<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByBoundAttribute<T>>
): ReturnType<FindAllByBoundAttribute<T>>
declare function getByRole<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByRole<T>>
): ReturnType<GetByRole<T>>
declare function getAllByRole<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByRole<T>>
): ReturnType<AllByRole<T>>
declare function queryByRole<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByRole<T>>
): ReturnType<QueryByRole<T>>
declare function queryAllByRole<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByRole<T>>
): ReturnType<AllByRole<T>>
declare function findByRole<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByRole<T>>
): ReturnType<FindByRole<T>>
declare function findAllByRole<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByRole<T>>
): ReturnType<FindAllByRole<T>>
declare function getByTestId<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<GetByBoundAttribute<T>>
): ReturnType<GetByBoundAttribute<T>>
declare function getAllByTestId<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function queryByTestId<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<QueryByBoundAttribute<T>>
): ReturnType<QueryByBoundAttribute<T>>
declare function queryAllByTestId<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<AllByBoundAttribute<T>>
): ReturnType<AllByBoundAttribute<T>>
declare function findByTestId<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindByBoundAttribute<T>>
): ReturnType<FindByBoundAttribute<T>>
declare function findAllByTestId<T extends HTMLElement = HTMLElement>(
  ...args: Parameters<FindAllByBoundAttribute<T>>
): ReturnType<FindAllByBoundAttribute<T>>

type queries_QueryByBoundAttribute<T extends HTMLElement = HTMLElement> = QueryByBoundAttribute<T>;
type queries_AllByBoundAttribute<T extends HTMLElement = HTMLElement> = AllByBoundAttribute<T>;
type queries_FindAllByBoundAttribute<T extends HTMLElement = HTMLElement> = FindAllByBoundAttribute<T>;
type queries_GetByBoundAttribute<T extends HTMLElement = HTMLElement> = GetByBoundAttribute<T>;
type queries_FindByBoundAttribute<T extends HTMLElement = HTMLElement> = FindByBoundAttribute<T>;
type queries_QueryByText<T extends HTMLElement = HTMLElement> = QueryByText<T>;
type queries_AllByText<T extends HTMLElement = HTMLElement> = AllByText<T>;
type queries_FindAllByText<T extends HTMLElement = HTMLElement> = FindAllByText<T>;
type queries_GetByText<T extends HTMLElement = HTMLElement> = GetByText<T>;
type queries_FindByText<T extends HTMLElement = HTMLElement> = FindByText<T>;
type queries_ByRoleOptions = ByRoleOptions;
type queries_AllByRole<T extends HTMLElement = HTMLElement> = AllByRole<T>;
type queries_GetByRole<T extends HTMLElement = HTMLElement> = GetByRole<T>;
type queries_QueryByRole<T extends HTMLElement = HTMLElement> = QueryByRole<T>;
type queries_FindByRole<T extends HTMLElement = HTMLElement> = FindByRole<T>;
type queries_FindAllByRole<T extends HTMLElement = HTMLElement> = FindAllByRole<T>;
declare const queries_getByLabelText: typeof getByLabelText;
declare const queries_getAllByLabelText: typeof getAllByLabelText;
declare const queries_queryByLabelText: typeof queryByLabelText;
declare const queries_queryAllByLabelText: typeof queryAllByLabelText;
declare const queries_findByLabelText: typeof findByLabelText;
declare const queries_findAllByLabelText: typeof findAllByLabelText;
declare const queries_getByPlaceholderText: typeof getByPlaceholderText;
declare const queries_getAllByPlaceholderText: typeof getAllByPlaceholderText;
declare const queries_queryByPlaceholderText: typeof queryByPlaceholderText;
declare const queries_queryAllByPlaceholderText: typeof queryAllByPlaceholderText;
declare const queries_findByPlaceholderText: typeof findByPlaceholderText;
declare const queries_findAllByPlaceholderText: typeof findAllByPlaceholderText;
declare const queries_getByText: typeof getByText;
declare const queries_getAllByText: typeof getAllByText;
declare const queries_queryByText: typeof queryByText;
declare const queries_queryAllByText: typeof queryAllByText;
declare const queries_findByText: typeof findByText;
declare const queries_findAllByText: typeof findAllByText;
declare const queries_getByAltText: typeof getByAltText;
declare const queries_getAllByAltText: typeof getAllByAltText;
declare const queries_queryByAltText: typeof queryByAltText;
declare const queries_queryAllByAltText: typeof queryAllByAltText;
declare const queries_findByAltText: typeof findByAltText;
declare const queries_findAllByAltText: typeof findAllByAltText;
declare const queries_getByTitle: typeof getByTitle;
declare const queries_getAllByTitle: typeof getAllByTitle;
declare const queries_queryByTitle: typeof queryByTitle;
declare const queries_queryAllByTitle: typeof queryAllByTitle;
declare const queries_findByTitle: typeof findByTitle;
declare const queries_findAllByTitle: typeof findAllByTitle;
declare const queries_getByDisplayValue: typeof getByDisplayValue;
declare const queries_getAllByDisplayValue: typeof getAllByDisplayValue;
declare const queries_queryByDisplayValue: typeof queryByDisplayValue;
declare const queries_queryAllByDisplayValue: typeof queryAllByDisplayValue;
declare const queries_findByDisplayValue: typeof findByDisplayValue;
declare const queries_findAllByDisplayValue: typeof findAllByDisplayValue;
declare const queries_getByRole: typeof getByRole;
declare const queries_getAllByRole: typeof getAllByRole;
declare const queries_queryByRole: typeof queryByRole;
declare const queries_queryAllByRole: typeof queryAllByRole;
declare const queries_findByRole: typeof findByRole;
declare const queries_findAllByRole: typeof findAllByRole;
declare const queries_getByTestId: typeof getByTestId;
declare const queries_getAllByTestId: typeof getAllByTestId;
declare const queries_queryByTestId: typeof queryByTestId;
declare const queries_queryAllByTestId: typeof queryAllByTestId;
declare const queries_findByTestId: typeof findByTestId;
declare const queries_findAllByTestId: typeof findAllByTestId;
declare namespace queries {
  export {
    queries_QueryByBoundAttribute as QueryByBoundAttribute,
    queries_AllByBoundAttribute as AllByBoundAttribute,
    queries_FindAllByBoundAttribute as FindAllByBoundAttribute,
    queries_GetByBoundAttribute as GetByBoundAttribute,
    queries_FindByBoundAttribute as FindByBoundAttribute,
    queries_QueryByText as QueryByText,
    queries_AllByText as AllByText,
    queries_FindAllByText as FindAllByText,
    queries_GetByText as GetByText,
    queries_FindByText as FindByText,
    queries_ByRoleOptions as ByRoleOptions,
    queries_AllByRole as AllByRole,
    queries_GetByRole as GetByRole,
    queries_QueryByRole as QueryByRole,
    queries_FindByRole as FindByRole,
    queries_FindAllByRole as FindAllByRole,
    queries_getByLabelText as getByLabelText,
    queries_getAllByLabelText as getAllByLabelText,
    queries_queryByLabelText as queryByLabelText,
    queries_queryAllByLabelText as queryAllByLabelText,
    queries_findByLabelText as findByLabelText,
    queries_findAllByLabelText as findAllByLabelText,
    queries_getByPlaceholderText as getByPlaceholderText,
    queries_getAllByPlaceholderText as getAllByPlaceholderText,
    queries_queryByPlaceholderText as queryByPlaceholderText,
    queries_queryAllByPlaceholderText as queryAllByPlaceholderText,
    queries_findByPlaceholderText as findByPlaceholderText,
    queries_findAllByPlaceholderText as findAllByPlaceholderText,
    queries_getByText as getByText,
    queries_getAllByText as getAllByText,
    queries_queryByText as queryByText,
    queries_queryAllByText as queryAllByText,
    queries_findByText as findByText,
    queries_findAllByText as findAllByText,
    queries_getByAltText as getByAltText,
    queries_getAllByAltText as getAllByAltText,
    queries_queryByAltText as queryByAltText,
    queries_queryAllByAltText as queryAllByAltText,
    queries_findByAltText as findByAltText,
    queries_findAllByAltText as findAllByAltText,
    queries_getByTitle as getByTitle,
    queries_getAllByTitle as getAllByTitle,
    queries_queryByTitle as queryByTitle,
    queries_queryAllByTitle as queryAllByTitle,
    queries_findByTitle as findByTitle,
    queries_findAllByTitle as findAllByTitle,
    queries_getByDisplayValue as getByDisplayValue,
    queries_getAllByDisplayValue as getAllByDisplayValue,
    queries_queryByDisplayValue as queryByDisplayValue,
    queries_queryAllByDisplayValue as queryAllByDisplayValue,
    queries_findByDisplayValue as findByDisplayValue,
    queries_findAllByDisplayValue as findAllByDisplayValue,
    queries_getByRole as getByRole,
    queries_getAllByRole as getAllByRole,
    queries_queryByRole as queryByRole,
    queries_queryAllByRole as queryAllByRole,
    queries_findByRole as findByRole,
    queries_findAllByRole as findAllByRole,
    queries_getByTestId as getByTestId,
    queries_getAllByTestId as getAllByTestId,
    queries_queryByTestId as queryByTestId,
    queries_queryAllByTestId as queryAllByTestId,
    queries_findByTestId as findByTestId,
    queries_findAllByTestId as findAllByTestId,
  };
}

type ElementToElementHandle<Input> = Input extends Element ? ElementHandle<Input> : Input extends (Element | ElementHandle)[] ? {
    [K in keyof Input]: ElementToElementHandle<Input[K]>;
} : Input;
type Promisify$1<Input> = Input extends Promise<any> ? Input : Promise<Input>;
type ValueOf<Input> = Input extends any[] ? Input[number] : Input[keyof Input];
type UnArray<Input> = Input extends any[] ? Input[number] : Input;
type UnPromise<Input> = Input extends Promise<infer Inner> ? Inner : Input;
/**
 * Changes type signature of an original testing library query function by:
 * - Removing the `container` parameter
 * - Returning a promise, always
 * - Returning ElementHandles instead of Elements
 */
type ChangeDTLFn<DTLFn extends ValueOf<typeof queries>> = DTLFn extends (container: HTMLElement, ...args: infer Args) => infer DTLReturn ? <CustomizedReturn extends UnArray<UnPromise<DTLReturn>>>(...args: Args) => Promisify$1<ElementToElementHandle<UnPromise<DTLReturn> extends any[] ? CustomizedReturn[] : CustomizedReturn>> : never;
type BoundQueries = {
    [K in keyof typeof queries]: ChangeDTLFn<typeof queries[K]>;
};
interface WaitForOptions {
    /**
     * The element watched by the MutationObserver which,
     * when it or its descendants change,
     * causes the callback to run again (regardless of the interval).
     * Default: `document.documentElement` (root element)
     */
    container?: ElementHandle;
    /**
     * The amount of time (milliseconds) that will pass before waitFor "gives up" and throws whatever the callback threw.
     * Default: 1000ms
     */
    timeout?: number;
    /**
     * The maximum amount of time (milliseconds) that will pass between each run of the callback.
     * If the MutationObserver notices a DOM change before this interval triggers,
     * the callback will run again immediately.
     * Default: 50ms
     */
    interval?: number;
    /** Manipulate the error thrown when the timeout triggers. */
    onTimeout?: (error: Error) => Error;
    /** Options to pass to initialize the MutationObserver. */
    mutationObserverOptions?: MutationObserverInit;
}

interface PleasantestUser {
    /** Clicks an element, if the element is visible and not covered by another element */
    click(element: ElementHandle | null, options?: {
        force?: boolean;
    } & Pick<UserOpts, 'targetSize'>): Promise<void>;
    /** Types text into an element, if the element is visible. The element must be an `<input>` or `<textarea>` or have `[contenteditable]`. */
    type(element: ElementHandle | null, text: string, options?: {
        delay?: number;
        force?: boolean;
    }): Promise<void>;
    /** Clears a text input's value, if the element is visible. The element must be an `<input>` or `<textarea>`. */
    clear(element: ElementHandle | null, options?: {
        force?: boolean;
    }): Promise<void>;
    /** Selects the specified option(s) of a <select> or a <select multiple> element. Values can be passed as either strings (option values) or as ElementHandle references to elements. */
    selectOptions(element: ElementHandle | null, values: ElementHandle | ElementHandle[] | string[] | string, options?: {
        force?: boolean;
    }): Promise<void>;
}
interface UserOpts {
    /**
     * Control the minimum target size, only for user.click
     * Pass `false` to disable the check
     */
    targetSize?: boolean | number;
}

declare global {
    namespace jest {
        interface Matchers<R> {
            /**
             * Check whether an element is disabled from the user's perspective.
             * https://github.com/testing-library/jest-dom#tobedisabled
             */
            toBeDisabled(): Promise<R>;
            /**
             * Check whether an element is not disabled from the user's perspective.
             * https://github.com/testing-library/jest-dom#tobeenabled
             * Same as .not.toBeDisabled()
             */
            toBeEnabled(): Promise<R>;
            /**
             * Assert whether an element has content or not.
             * https://github.com/testing-library/jest-dom#tobeemptydomelement
             */
            toBeEmptyDOMElement(): Promise<R>;
            /**
             * Assert whether an element is present in the document or not.
             * https://github.com/testing-library/jest-dom#tobeinthedocument
             */
            toBeInTheDocument(): Promise<R>;
            /**
             * Check if the value of an element is currently invalid.
             * Uses [HTML5 Constraint Validation](https://developer.mozilla.org/en-US/docs/Web/Guide/HTML/HTML5/Constraint_validation) and checks for `aria-invalid`.
             * https://github.com/testing-library/jest-dom#tobeinvalid
             */
            toBeInvalid(): Promise<R>;
            /**
             * Check if a form element is currently required.
             * https://github.com/testing-library/jest-dom#toberequired
             */
            toBeRequired(): Promise<R>;
            /**
             * Check if the value of an element is currently valid.
             * Uses [HTML5 Constraint Validation](https://developer.mozilla.org/en-US/docs/Web/Guide/HTML/HTML5/Constraint_validation) and checks for `aria-invalid`.
             * https://github.com/testing-library/jest-dom#tobevalid
             */
            toBeValid(): Promise<R>;
            /**
             * Check if an element is currently visible to the user.
             * https://github.com/testing-library/jest-dom#tobevisible
             */
            toBeVisible(): Promise<R>;
            /**
             * Check if an element contains another element as a descendant.
             * https://github.com/testing-library/jest-dom#tocontainelement
             */
            toContainElement(element: ElementHandle<HTMLElement | SVGElement> | null): Promise<R>;
            /**
             * Check whether a string representing a HTML element is contained in another element.
             * https://github.com/testing-library/jest-dom#tocontainhtml
             */
            toContainHTML(html: string): Promise<R>;
            /**
             * Assert that an element has the expected [accessible description](https://www.w3.org/TR/accname-1.1/#dfn-accessible-description).
             * You can pass the exact string, or you can make a partial match passing a regular expression
             * https://github.com/testing-library/jest-dom#tohaveaccessibledescription
             */
            toHaveAccessibleDescription(text?: string | RegExp): Promise<R>;
            /**
             * Assert that an element has the expected [accessible description](https://www.w3.org/TR/accname-1.1/#dfn-accessible-name).
             * It is useful, for instance, to assert that form elements and buttons are properly labelled.
             * You can pass the exact string, or you can make a partial match passing a regular expression
             * https://github.com/testing-library/jest-dom#tohaveaccessibledescription
             */
            toHaveAccessibleName(text?: string | RegExp): Promise<R>;
            /**
             * Check whether the given element has an attribute or not.
             * You can also optionally check that the attribute has a specific expected value
             * https://github.com/testing-library/jest-dom#tohaveattribute
             */
            toHaveAttribute(attr: string, value?: string): Promise<R>;
            /**
             * Check whether the given element has certain classes within its class attribute.
             * You must provide at least one class, unless you are asserting that an element does not have any classes.
             * https://github.com/testing-library/jest-dom#tohaveclass
             */
            toHaveClass(...classNames: string[]): Promise<R>;
            toHaveClass(classNames: string, options?: {
                exact: boolean;
            }): Promise<R>;
            /**
             * Check whether an element has focus
             * https://github.com/testing-library/jest-dom#tohavefocus
             */
            toHaveFocus(): Promise<R>;
            /**
             * Check if a form or fieldset contains form controls for each given name, and value.
             * https://github.com/testing-library/jest-dom#tohaveformvalues
             */
            toHaveFormValues(expectedValues: Record<string, unknown>): Promise<R>;
            /**
             * Check if an element has specific css properties applied
             * Unlike jest-dom, pleasantest does not support specifying expected styles as strings, they must be specified as an object.
             * https://github.com/testing-library/jest-dom#tohavestyle
             */
            toHaveStyle(css: Record<string, unknown>): Promise<R>;
            /**
             * Check whether the given element has a text content
             * https://github.com/testing-library/jest-dom#tohavetextcontent
             */
            toHaveTextContent(text: string | RegExp, options?: {
                normalizeWhitespace: boolean;
            }): Promise<R>;
            /**
             * Check whether the given form element has the specified value.
             * It accepts <input>, <select> and <textarea> elements with the exception of <input type="checkbox"> and <input type="radio">, which should be matched using toBeChecked or toHaveFormValues.
             * https://github.com/testing-library/jest-dom#tohavevalue
             */
            toHaveValue(value?: string | string[] | number | null): Promise<R>;
            /**
             * Check whether the given form element has the specified displayed value (the one the end user will see).
             * It accepts <input>, <select> and <textarea> elements with the exception of <input type="checkbox"> and <input type="radio">, which should be matched using toBeChecked or toHaveFormValues.
             * https://github.com/testing-library/jest-dom#tohavedisplayvalue
             */
            toHaveDisplayValue(value: string | RegExp | (string | RegExp)[]): Promise<R>;
            /**
             * Check whether the given element is checked.
             * Accepts an `input` of type `checkbox` or `radio`
             * and elements with a `role` of `checkbox`, `radio` or `switch`
             * with a valid `aria-checked` attribute of "true" or "false".
             * https://github.com/testing-library/jest-dom#tobechecked
             */
            toBeChecked(): Promise<R>;
            /**
             * Check whether the given element is partially checked.
             * It accepts an `input` of type `checkbox` and elements with a `role` of `checkbox` with `aria-checked="mixed"`, or input of type `checkbox` with `indeterminate` set to `true`
             
             * https://github.com/testing-library/jest-dom#tobepartiallychecked
             */
            toBePartiallyChecked(): Promise<R>;
            /**
             * Check whether the given element has an ARIA error message (via aria-errormessage)
             * https://github.com/testing-library/jest-dom#tohaveerrormessage
             */
            toHaveErrorMessage(text: string | RegExp): Promise<R>;
        }
    }
}

/**
 * Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */


declare type Colors = {
  comment: {
    close: string;
    open: string;
  };
  content: {
    close: string;
    open: string;
  };
  prop: {
    close: string;
    open: string;
  };
  tag: {
    close: string;
    open: string;
  };
  value: {
    close: string;
    open: string;
  };
};

declare type CompareKeys =
  | ((a: string, b: string) => number)
  | null
  | undefined;

declare type Config = {
  callToJSON: boolean;
  compareKeys: CompareKeys;
  colors: Colors;
  escapeRegex: boolean;
  escapeString: boolean;
  indent: string;
  maxDepth: number;
  maxWidth: number;
  min: boolean;
  plugins: Plugins;
  printBasicPrototype: boolean;
  printFunctionName: boolean;
  spacingInner: string;
  spacingOuter: string;
};

declare type Indent = (arg0: string) => string;

declare type NewPlugin = {
  serialize: (
    val: any,
    config: Config,
    indentation: string,
    depth: number,
    refs: Refs,
    printer: Printer,
  ) => string;
  test: Test;
};

declare type OldPlugin = {
  print: (
    val: unknown,
    print: Print,
    indent: Indent,
    options: PluginOptions,
    colors: Colors,
  ) => string;
  test: Test;
};

declare type Plugin_2 = NewPlugin | OldPlugin;


declare type PluginOptions = {
  edgeSpacing: string;
  min: boolean;
  spacing: string;
};

declare type Plugins = Array<Plugin_2>;

declare type Print = (arg0: unknown) => string;

declare type Printer = (
  val: unknown,
  config: Config,
  indentation: string,
  depth: number,
  refs: Refs,
  hasCalledToJSON?: boolean,
) => string;

declare type Refs = Array<unknown>;

declare type Test = (arg0: any) => boolean;

declare const accessibilityTreeSymbol: unique symbol;
interface AccessibilityTreeOptions {
    /**
     * Whether the accessible description of elements should be included in the tree.
     * https://www.w3.org/TR/wai-aria-1.2/#dfn-accessible-description
     * (default: true)
     */
    includeDescriptions?: boolean;
    /**
     * Whether to include text that is not being used as an element's name
     * (default: true)
     */
    includeText?: boolean;
}
declare const getAccessibilityTree: (element: ElementHandle | Page, options?: AccessibilityTreeOptions) => Promise<{
    [accessibilityTreeSymbol]: string;
    toString(): string;
} | undefined>;
declare const getAccessibilityTreeWrapper: typeof getAccessibilityTree;

declare const accessibilityTreeSnapshotSerializer: NewPlugin;
interface ToPassAxeTestsOpts {
    /** CSS selector(s) to add to the list of elements to include in analysis. */
    include?: string | string[];
    /** CSS selector(s) to add to the list of elements to exclude from analysis. */
    exclude?: string | string[];
    /** The list of Axe rules to skip from verification. */
    disabledRules?: string | string[];
    /** A flexible way to configure how Axe run operates, see https://github.com/dequelabs/axe-core/blob/HEAD/doc/API.md#options-parameter. */
    options?: axe.RunOptions;
    /** Axe configuration object, see https://github.com/dequelabs/axe-core/blob/HEAD/doc/API.md#api-name-axeconfigure. */
    config?: axe.Spec;
}
declare global {
    namespace jest {
        interface Matchers<R> {
            toPassAxeTests(opts?: ToPassAxeTestsOpts): Promise<R>;
        }
    }
}

type Promisify<T> = T extends Promise<any> ? T : Promise<T>;
type JSHandleify<T> = T extends JSHandle<any> ? T : JSHandle<T>;
/**
 * Wraps a JSHandle that points to a function in a browser,
 * with a node function that calls the corresponding browser function,
 * passing along the parameters,
 * and returning the return value wrapped in `Promise<JSHandle<T>>`
 */
declare const makeCallableJSHandle: <BrowserFunction extends (...args: any[]) => any>(browserFunction: JSHandle<BrowserFunction>) => (...args: Parameters<BrowserFunction>) => Promisify<JSHandleify<ReturnType<BrowserFunction>>>;

interface PleasantestUtils {
    /**
     * Execute a JS code string in the browser.
     * The code string inherits the syntax abilities of the file it is in,
     * i.e. if your test file is a .tsx file, then the code string can include JSX and TS.
     * The code string can use (static or dynamic) ES6 imports to import other modules,
     * including TS/JSX modules, and it supports resolving from node_modules,
     * and relative paths from the test file.
     * The code string supports top-level await to wait for a Promise to resolve.
     * You can pass an array of variables to be passed into the browser as the 2nd parameter.
     */
    runJS<Module extends Record<string, unknown>>(code: string, args?: unknown[]): Promise<{
        [Export in keyof Module]: puppeteer.JSHandle<Module[Export]>;
    }>;
    /** Set the contents of a new style tag */
    injectCSS(css: string): Promise<void>;
    /** Set the contents of document.body */
    injectHTML(html: string, opts?: {
        executeScriptTags?: boolean;
    }): Promise<void>;
    /** Load a CSS (or Sass, Less, etc.) file into the browser. Pass a path that will be resolved from your test file. */
    loadCSS(cssPath: string): Promise<void>;
    /** Load a JS (or TS, JSX) file into the browser. Pass a path that will be resolved from your test file. */
    loadJS(jsPath: string): Promise<void>;
}
interface PleasantestContext {
    /** DOM Testing Library queries that are bound to the document */
    screen: BoundQueries;
    utils: PleasantestUtils;
    /** Returns DOM Testing Library queries that only search within a single element */
    within(element: puppeteer.ElementHandle | null): BoundQueries;
    page: puppeteer.Page;
    user: PleasantestUser;
    waitFor: <T>(cb: () => T | Promise<T>, opts?: WaitForOptions) => Promise<T>;
}
interface WithBrowserOpts {
    headless?: boolean;
    device?: puppeteer.Device;
    moduleServer?: ModuleServerOpts;
    user?: UserOpts;
}
interface TestFn {
    (ctx: PleasantestContext): boolean | void | Promise<boolean | void>;
}
interface WithBrowserFn {
    (testFn: TestFn): () => Promise<void>;
    (options: WithBrowserOpts, testFn: TestFn): () => Promise<void>;
}
interface WithBrowser extends WithBrowserFn {
    headed: WithBrowserFn;
}
declare const withBrowser: WithBrowser;
declare const configureDefaults: (options: WithBrowserOpts) => void;
declare const devices: puppeteer.DevicesMap;

export { PleasantestContext, PleasantestUser, PleasantestUtils, WaitForOptions, WithBrowserOpts, accessibilityTreeSnapshotSerializer, configureDefaults, devices, getAccessibilityTreeWrapper as getAccessibilityTree, makeCallableJSHandle, withBrowser };
